@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -77,6 +77,7 @@
 #include <e32def.h>
 #include <e32debug.h>
 #include <f32file.h>
+#include <e32math.h>
 # include "private/qcore_symbian_p.h"
 
 _LIT(qt_S60Filter, "Series60v?.*.sis");
@@ -1225,7 +1226,7 @@ bool qSharedBuild()
 
     Defined on Mac OS X.
 
-    \sa Q_WS_WIN, Q_WS_X11, Q_WS_QWS
+    \sa Q_WS_WIN, Q_WS_X11, Q_WS_QWS, Q_WS_S60
 */
 
 /*!
@@ -1234,7 +1235,7 @@ bool qSharedBuild()
 
     Defined on Windows.
 
-    \sa Q_WS_MAC, Q_WS_X11, Q_WS_QWS
+    \sa Q_WS_MAC, Q_WS_X11, Q_WS_QWS, Q_WS_S60
 */
 
 /*!
@@ -1243,7 +1244,7 @@ bool qSharedBuild()
 
     Defined on X11.
 
-    \sa Q_WS_MAC, Q_WS_WIN, Q_WS_QWS
+    \sa Q_WS_MAC, Q_WS_WIN, Q_WS_QWS, Q_WS_S60
 */
 
 /*!
@@ -1252,7 +1253,7 @@ bool qSharedBuild()
 
     Defined on Qt for Embedded Linux.
 
-    \sa Q_WS_MAC, Q_WS_WIN, Q_WS_X11
+    \sa Q_WS_MAC, Q_WS_WIN, Q_WS_X11, Q_WS_S60
 */
 
 /*!
@@ -1598,6 +1599,29 @@ bool qSharedBuild()
     Optimizing C++ Compilers.
 */
 
+/*!
+  \macro Q_OS_MAC
+  \relates <QtGlobal>
+
+  Defined on MAC OS (synonym for Darwin).
+ */
+
+/*!
+  \macro Q_OS_SYMBIAN
+  \relates <QtGlobal>
+
+  Defined on Symbian.
+ */
+
+/*!
+  \macro Q_WS_S60
+  \relates <QtGlobal>
+
+  Defined on S60.
+
+  \sa Q_WS_MAC, Q_WS_WIN, Q_WS_X11, Q_WS_QWS
+ */
+
 #if defined(QT_BUILD_QMAKE)
 // needed to bootstrap qmake
 static const unsigned int qt_one = 1;
@@ -1970,7 +1994,6 @@ void qt_check_pointer(const char *n, int l)
     qWarning("In file %s, line %d: Out of memory", n, l);
 }
 
-#ifndef QT_NO_EXCEPTIONS
 /* \internal
    Allows you to throw an exception without including <new>
    Called internally from Q_CHECK_PTR on certain OS combinations
@@ -1979,7 +2002,6 @@ void qBadAlloc()
 {
     QT_THROW(std::bad_alloc());
 }
-#endif
 
 /*
   The Q_ASSERT macro calls this function when the test fails.
@@ -2052,7 +2074,28 @@ static void mac_default_handler(const char *msg)
 }
 #endif // Q_CC_MWERKS && Q_OS_MACX
 
-
+#if !defined(Q_OS_WIN) && !defined(QT_NO_THREAD) && !defined(Q_OS_INTEGRITY) && !defined(Q_OS_QNX) && \
+    defined(_POSIX_THREAD_SAFE_FUNCTIONS) && _POSIX_VERSION >= 200112L
+namespace {
+    // There are two incompatible versions of strerror_r:
+    // a) the XSI/POSIX.1 version, which returns an int,
+    //    indicating success or not
+    // b) the GNU version, which returns a char*, which may or may not
+    //    be the beginning of the buffer we used
+    // The GNU libc manpage for strerror_r says you should use the the XSI
+    // version in portable code. However, it's impossible to do that if
+    // _GNU_SOURCE is defined so we use C++ overloading to decide what to do
+    // depending on the return type
+    static inline QString fromstrerror_helper(int, const QByteArray &buf)
+    {
+        return QString::fromLocal8Bit(buf);
+    }
+    static inline QString fromstrerror_helper(const char *str, const QByteArray &)
+    {
+        return QString::fromLocal8Bit(str);
+    }
+}
+#endif
 
 QString qt_error_string(int errorCode)
 {
@@ -2095,12 +2138,9 @@ QString qt_error_string(int errorCode)
 
         if (ret.isEmpty() && errorCode == ERROR_MOD_NOT_FOUND)
             ret = QString::fromLatin1("The specified module could not be found.");
-
 #elif !defined(QT_NO_THREAD) && defined(_POSIX_THREAD_SAFE_FUNCTIONS) && _POSIX_VERSION >= 200112L && !defined(Q_OS_INTEGRITY) && !defined(Q_OS_QNX)
-
         QByteArray buf(1024, '\0');
-        strerror_r(errorCode, buf.data(), buf.size());
-        ret = QString::fromLocal8Bit(buf.constData());
+        ret = fromstrerror_helper(strerror_r(errorCode, buf.data(), buf.size()), buf);
 #else
         ret = QString::fromLocal8Bit(strerror(errorCode));
 #endif
@@ -2493,7 +2533,7 @@ bool qputenv(const char *varName, const QByteArray& value)
 #endif
 }
 
-#if (defined(Q_OS_UNIX) || defined(Q_OS_WIN)) && !defined(QT_NO_THREAD) && !defined(Q_OS_SYMBIAN)
+#if (defined(Q_OS_UNIX) || defined(Q_OS_WIN)) && !defined(QT_NO_THREAD)
 
 #  if defined(Q_OS_INTEGRITY) && defined(__GHS_VERSION_NUMBER) && (__GHS_VERSION_NUMBER < 500)
 // older versions of INTEGRITY used a long instead of a uint for the seed.
@@ -2516,8 +2556,6 @@ Q_GLOBAL_STATIC(SeedStorage, randTLS)  // Thread Local Storage for seed value
     Sets the argument \a seed to be used to generate a new random number sequence of
     pseudo random integers to be returned by qrand().
 
-    If no seed value is provided, qrand() is automatically seeded with a value of 1.
-
     The sequence of random numbers generated is deterministic per thread. For example,
     if two threads call qsrand(1) and subsequently calls qrand(), the threads will get
     the same random number sequence.
@@ -2541,8 +2579,9 @@ void qsrand(uint seed)
         srand(seed);
     }
 #else
-    // On Windows srand() and rand() already use Thread-Local-Storage
+    // On Windows and Symbian srand() and rand() already use Thread-Local-Storage
     // to store the seed between calls
+    // this is also valid for QT_NO_THREAD
     srand(seed);
 #endif
 }
@@ -2558,7 +2597,7 @@ void qsrand(uint seed)
 */
 void qsrand()
 {
-#if (defined(Q_OS_UNIX) || defined(Q_OS_WIN)) && !defined(QT_NO_THREAD) && !defined(Q_OS_SYMBIAN)
+#if (defined(Q_OS_UNIX) || defined(Q_OS_WIN)) && !defined(QT_NO_THREAD)
     SeedStorage *seedStorage = randTLS();
     if (seedStorage) {
         SeedStorageType *pseed = seedStorage->localData();
@@ -2572,24 +2611,28 @@ void qsrand()
         *pseed = QDateTime::currentDateTime().toTime_t()
                  + quintptr(&pseed)
                  + serial.fetchAndAddRelaxed(1);
-#if defined(Q_OS_WIN)
-        // for Windows the srand function must still be called.
+#if defined(Q_OS_WIN) || defined(Q_OS_SYMBIAN)
+        // for Windows and Symbian the srand function must still be called.
         srand(*pseed);
 #endif
     }
 
-#elif defined(Q_OS_WIN)
+//QT_NO_THREAD implementations
+#else
     static unsigned int seed = 0;
 
     if (seed)
         return;
 
+#if defined(Q_OS_SYMBIAN)
+    seed = Math::Random();
+#elif defined(Q_OS_WIN)
     seed = GetTickCount();
-    srand(seed);
 #else
-    // Symbian?
-
-#endif // defined(Q_OS_UNIX) || defined(Q_OS_WIN)) && !defined(QT_NO_THREAD) && !defined(Q_OS_SYMBIAN)
+    seed = quintptr(&seed) + QDateTime::currentDateTime().toTime_t();
+#endif
+    srand(seed);
+#endif // defined(Q_OS_UNIX) || defined(Q_OS_WIN)) && !defined(QT_NO_THREAD)
 }
 
 /*!
@@ -2626,8 +2669,9 @@ int qrand()
         return rand();
     }
 #else
-    // On Windows srand() and rand() already use Thread-Local-Storage
+    // On Windows and Symbian srand() and rand() already use Thread-Local-Storage
     // to store the seed between calls
+    // this is also valid for QT_NO_THREAD
     return rand();
 #endif
 }
