@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -68,6 +68,14 @@ extern uint qGlobalPostedEventsCount();
 #  define QS_RAWINPUT 0x0400
 #endif
 
+#ifndef WM_TOUCH
+#  define WM_TOUCH 0x0240
+#endif
+#ifndef WM_GESTURE
+#  define WM_GESTURE 0x0119
+#  define WM_GESTURENOTIFY 0x011A
+#endif
+
 enum {
     WM_QT_SOCKETNOTIFIER = WM_USER,
     WM_QT_SENDPOSTEDEVENTS = WM_USER + 1,
@@ -336,6 +344,7 @@ public:
 
     // internal window handle used for socketnotifiers/timers/etc
     HWND internalHwnd;
+    HHOOK getMessageHook;
 
     // for controlling when to send posted events
     QAtomicInt serialNumber;
@@ -363,7 +372,7 @@ public:
 };
 
 QEventDispatcherWin32Private::QEventDispatcherWin32Private()
-    : threadId(GetCurrentThreadId()), interrupt(false), internalHwnd(0), serialNumber(0), lastSerialNumber(0), wakeUps(0)
+    : threadId(GetCurrentThreadId()), interrupt(false), internalHwnd(0), getMessageHook(0), serialNumber(0), lastSerialNumber(0), wakeUps(0)
 {
     resolveTimerAPI();
 }
@@ -471,37 +480,11 @@ LRESULT CALLBACK qt_internal_proc(HWND hwnd, UINT message, WPARAM wp, LPARAM lp)
         }
         return 0;
     } else if (message == WM_TIMER) {    
-        if (wp == SendPostedEventsTimerId) {
-            KillTimer(d->internalHwnd, wp);
-            int localSerialNumber = d->serialNumber;
-            (void) d->wakeUps.fetchAndStoreRelease(0);
-            if (localSerialNumber != d->lastSerialNumber) {
-                PostMessage(d->internalHwnd, WM_QT_SENDPOSTEDEVENTS, 0, 0);
-            }
-        } else {
-            Q_ASSERT(d != 0);
-            d->sendTimerEvent(wp);
-        }
+        Q_ASSERT(d != 0);
+        d->sendTimerEvent(wp);
         return 0;
     } else if (message == WM_QT_SENDPOSTEDEVENTS) {
         int localSerialNumber = d->serialNumber;
-
-        if (GetQueueStatus(QS_INPUT | QS_RAWINPUT | QS_TIMER) != 0) {
-            // delay the next pass of sendPostedEvents() until we get the special
-            // WM_TIMER, which allows all pending Windows messages to be processed
-            if (SetTimer(d->internalHwnd, SendPostedEventsTimerId, 0, 0) == 0) {
-                // failed to start the timer, oops, clear wakeUps in an attempt to keep things running
-                qErrnoWarning("Qt: INTERNAL ERROR: failed to start sendPostedEvents() timer");
-                d->wakeUps.fetchAndStoreRelease(0);
-            } else {
-                // SetTimer() succeeded, nothing to do now
-                ;
-            }
-        } else {
-            // nothing pending in the queue, let sendPostedEvents go through
-            d->wakeUps.fetchAndStoreRelease(0);
-        }
-
         if (localSerialNumber != d->lastSerialNumber) {
             d->lastSerialNumber = localSerialNumber;
             QCoreApplicationPrivate::sendPostedEvents(0, 0, d->threadData);
@@ -512,6 +495,35 @@ LRESULT CALLBACK qt_internal_proc(HWND hwnd, UINT message, WPARAM wp, LPARAM lp)
     return DefWindowProc(hwnd, message, wp, lp);
 }
 
+LRESULT CALLBACK qt_GetMessageHook(int code, WPARAM wp, LPARAM lp)
+{
+    if (wp == PM_REMOVE) {
+        QEventDispatcherWin32 *q = qobject_cast<QEventDispatcherWin32 *>(QAbstractEventDispatcher::instance());
+        Q_ASSERT(q != 0);
+        if (q) {
+            QEventDispatcherWin32Private *d = q->d_func();
+            int localSerialNumber = d->serialNumber;
+            if (HIWORD(GetQueueStatus(QS_INPUT | QS_RAWINPUT | QS_TIMER)) == 0) {
+                // no more input or timer events in the message queue, we can allow posted events to be
+                // sent now
+                (void) d->wakeUps.fetchAndStoreRelease(0);
+                MSG *msg = (MSG *) lp;
+                if (localSerialNumber != d->lastSerialNumber
+                    // if this message IS the one that triggers sendPostedEvents(), no need to post it again
+                    && (msg->hwnd != d->internalHwnd
+                        || msg->message != WM_QT_SENDPOSTEDEVENTS)) {
+                    PostMessage(d->internalHwnd, WM_QT_SENDPOSTEDEVENTS, 0, 0);
+                }
+            }
+        }
+    }
+#ifdef Q_OS_WINCE
+    return 0;
+#else
+    return CallNextHookEx(0, code, wp, lp);
+#endif
+}
+
 static HWND qt_create_internal_window(const QEventDispatcherWin32 *eventDispatcher)
 {
     // make sure that multiple Qt's can coexist in the same process
@@ -636,6 +648,14 @@ void QEventDispatcherWin32::createInternalHwnd()
         return;
     d->internalHwnd = qt_create_internal_window(this);
 
+#ifndef Q_OS_WINCE
+    // setup GetMessage hook needed to drive our posted events
+    d->getMessageHook = SetWindowsHookEx(WH_GETMESSAGE, (HOOKPROC) qt_GetMessageHook, NULL, GetCurrentThreadId());
+    if (!d->getMessageHook) {
+        qFatal("Qt: INTERNALL ERROR: failed to install GetMessage hook");
+    }
+#endif
+
     // register all socket notifiers
     QList<int> sockets = (d->sn_read.keys().toSet()
                           + d->sn_write.keys().toSet()
@@ -702,6 +722,9 @@ bool QEventDispatcherWin32::processEvents(QEventLoop::ProcessEventsFlags flags)
                             && msg.message <= WM_MOUSELAST)
                         || msg.message == WM_MOUSEWHEEL
                         || msg.message == WM_MOUSEHWHEEL
+                        || msg.message == WM_TOUCH
+                        || msg.message == WM_GESTURE
+                        || msg.message == WM_GESTURENOTIFY
                         || msg.message == WM_CLOSE)) {
                     // queue user input events for later processing
                     haveMessage = false;
@@ -725,6 +748,11 @@ bool QEventDispatcherWin32::processEvents(QEventLoop::ProcessEventsFlags flags)
                 }
             }
             if (haveMessage) {
+#ifdef Q_OS_WINCE
+                // WinCE doesn't support hooks at all, so we have to call this by hand :(
+                (void) qt_GetMessageHook(0, PM_REMOVE, (LPARAM) &msg);
+#endif
+
                 if (d->internalHwnd == msg.hwnd && msg.message == WM_QT_SENDPOSTEDEVENTS) {
                     if (seenWM_QT_SENDPOSTEDEVENTS) {
                         needWM_QT_SENDPOSTEDEVENTS = true;
@@ -1058,6 +1086,12 @@ void QEventDispatcherWin32::closingDown()
         d->unregisterTimer(d->timerVec.at(i), true);
     d->timerVec.clear();
     d->timerDict.clear();
+
+#ifndef Q_OS_WINCE
+    if (d->getMessageHook)
+        UnhookWindowsHookEx(d->getMessageHook);
+    d->getMessageHook = 0;
+#endif
 }
 
 bool QEventDispatcherWin32::event(QEvent *e)
